# Introduction

This repo holds code for running benchmarks used in the paper:

Nicolas Blin, Edwin Carlinet, Florian Lemaitre, Lionel Lacassagne, Thierry Géraud. Max-tree Computation on GPUs. 2022. ⟨hal-03556296⟩

It provides many GPU and CPU implementations for the max-tree construction as well as many GPU variations. This code
should only be used to reproduce the benchmarks, it does not have a stable API (we are currently working on a more
stable API with only GPU's code).


# Building

0. Add pylene repository to conan: `conan remote add lrde-public https://artifactory.lrde.epita.fr/artifactory/api/conan/lrde-public`
1. Install dependencies with the conan C++ package manager. If ``conan`` is installed for the first time, you should
   configure it to use the C++-11 ABI.
    `conan profile update settings.compiler.libcxx=libstdc++11 default`
2. Set the path to the CUDA compiler: ``export CUDACXX=/usr/local/cuda-11.2/bin/nvcc``
3. Build with cmake, and ninja (or make)

```shell
> mkdir build && cd build
> conan install .. -s compiler.cppstd=20 -g cmake --build missing
> cmake .. -DCMAKE_BUILD_TYPE=Release -G Ninja
> ninja
```

4. See the folder "./bin"

# Running the benchmarks

1. Running benchmarks with memory transfers on:

```shell
>  ./bin/TestMaxtree --no-check  ./BHdV_PL_ATL20Ardt_1926_0004.pgm
```

2. Running benchmarks with memory transfers off:

```shell
>  ./bin/TestMaxtree --no-check --bench-kernel-only ./BHdV_PL_ATL20Ardt_1926_0004.pgm
```

3. Terminology

"Test_name"/"connectivity"/["C8_OPTIM"] (0 : no C8 optim ; 256 : C8 optim)

(See [Google Benchmark](https://github.com/google/benchmark/blob/main/docs/user_guide.md#running-benchmarks) to see more options when running the benchmarks)


# Test images

The 3 test images can be found here:

https://cloud.lrde.epita.fr/s/a4SKN4ZfpepQMpc

